// Based on dune/tnnmg/problem-classes/convexproblem.hh
// Allows phi to be const; does away with everything we do not need

#ifndef MY_CONVEX_PROBLEM_HH
#define MY_CONVEX_PROBLEM_HH

#include "globalnonlinearity.hh"

template <class MatrixTypeTEMPLATE, class VectorTypeTEMPLATE>
class MyConvexProblem {
public:
  typedef VectorTypeTEMPLATE VectorType;
  typedef MatrixTypeTEMPLATE MatrixType;
  typedef typename VectorType::block_type LocalVectorType;
  typedef typename MatrixType::block_type LocalMatrixType;

  int static const block_size = VectorType::block_type::dimension;

  /** \brief Constructor with the problem components

      \param A The matrix of the quadratic part
      \param f The linear functional
      \param u The solution vector
  */
  MyConvexProblem(MatrixType const &A,
                  Dune::GlobalNonlinearity<MatrixType, VectorType> const &phi,
                  VectorType const &f)
      : A(A), phi(phi), f(f) {}

  /* Just for debugging */
  double operator()(VectorType const &x) const {
    double ret = phi(x) - (f * x);
    VectorType tmp(x.size());
    A.mv(x, tmp);
    return ret + 0.5 * (tmp * x);
  }

  MatrixType const &A;
  Dune::GlobalNonlinearity<MatrixType, VectorType> const &phi;
  VectorType const &f;
};

#endif
